@@ -21,21 +21,22 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "../curl_setup.h"
 
 #include "warnless.h"
 
 #if defined(__INTEL_COMPILER) && defined(__unix__)
 
 #ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
+#  include <netinet/in.h>
 #endif
 #ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
+#  include <arpa/inet.h>
 #endif
 
 #endif /* __INTEL_COMPILER && __unix__ */
 
+#include <limits.h>
+
 #define CURL_MASK_UCHAR   ((unsigned char)~0)
 
 #define CURL_MASK_USHORT  ((unsigned short)~0)
@@ -55,15 +56,15 @@
 unsigned char curlx_ultouc(unsigned long ulnum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
-  DEBUGASSERT(ulnum <= (unsigned long)CURL_MASK_UCHAR);
-  return (unsigned char)(ulnum & (unsigned long)CURL_MASK_UCHAR);
+  DEBUGASSERT(ulnum <= (unsigned long) CURL_MASK_UCHAR);
+  return (unsigned char)(ulnum & (unsigned long) CURL_MASK_UCHAR);
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -74,15 +75,15 @@ unsigned char curlx_ultouc(unsigned long ulnum)
 int curlx_uztosi(size_t uznum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
-  DEBUGASSERT(uznum <= (size_t)CURL_MASK_SINT);
-  return (int)(uznum & (size_t)CURL_MASK_SINT);
+  DEBUGASSERT(uznum <= (size_t) CURL_MASK_SINT);
+  return (int)(uznum & (size_t) CURL_MASK_SINT);
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -93,17 +94,17 @@ int curlx_uztosi(size_t uznum)
 unsigned long curlx_uztoul(size_t uznum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
 #if ULONG_MAX < SIZE_MAX
-  DEBUGASSERT(uznum <= (size_t)CURL_MASK_ULONG);
+  DEBUGASSERT(uznum <= (size_t) CURL_MASK_ULONG);
 #endif
-  return (unsigned long)(uznum & (size_t)CURL_MASK_ULONG);
+  return (unsigned long)(uznum & (size_t) CURL_MASK_ULONG);
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -114,17 +115,17 @@ unsigned long curlx_uztoul(size_t uznum)
 unsigned int curlx_uztoui(size_t uznum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
 #if UINT_MAX < SIZE_MAX
-  DEBUGASSERT(uznum <= (size_t)CURL_MASK_UINT);
+  DEBUGASSERT(uznum <= (size_t) CURL_MASK_UINT);
 #endif
-  return (unsigned int)(uznum & (size_t)CURL_MASK_UINT);
+  return (unsigned int)(uznum & (size_t) CURL_MASK_UINT);
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -135,18 +136,18 @@ unsigned int curlx_uztoui(size_t uznum)
 int curlx_sltosi(long slnum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
   DEBUGASSERT(slnum >= 0);
 #if INT_MAX < LONG_MAX
-  DEBUGASSERT((unsigned long)slnum <= (unsigned long)CURL_MASK_SINT);
+  DEBUGASSERT((unsigned long) slnum <= (unsigned long) CURL_MASK_SINT);
 #endif
-  return (int)(slnum & (long)CURL_MASK_SINT);
+  return (int)(slnum & (long) CURL_MASK_SINT);
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -157,18 +158,18 @@ int curlx_sltosi(long slnum)
 unsigned int curlx_sltoui(long slnum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
   DEBUGASSERT(slnum >= 0);
 #if UINT_MAX < LONG_MAX
-  DEBUGASSERT((unsigned long)slnum <= (unsigned long)CURL_MASK_UINT);
+  DEBUGASSERT((unsigned long) slnum <= (unsigned long) CURL_MASK_UINT);
 #endif
-  return (unsigned int)(slnum & (long)CURL_MASK_UINT);
+  return (unsigned int)(slnum & (long) CURL_MASK_UINT);
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -179,16 +180,16 @@ unsigned int curlx_sltoui(long slnum)
 unsigned short curlx_sltous(long slnum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
   DEBUGASSERT(slnum >= 0);
-  DEBUGASSERT((unsigned long)slnum <= (unsigned long)CURL_MASK_USHORT);
-  return (unsigned short)(slnum & (long)CURL_MASK_USHORT);
+  DEBUGASSERT((unsigned long) slnum <= (unsigned long) CURL_MASK_USHORT);
+  return (unsigned short)(slnum & (long) CURL_MASK_USHORT);
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -199,15 +200,15 @@ unsigned short curlx_sltous(long slnum)
 ssize_t curlx_uztosz(size_t uznum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
-  DEBUGASSERT(uznum <= (size_t)CURL_MASK_SSIZE_T);
-  return (ssize_t)(uznum & (size_t)CURL_MASK_SSIZE_T);
+  DEBUGASSERT(uznum <= (size_t) CURL_MASK_SSIZE_T);
+  return (ssize_t)(uznum & (size_t) CURL_MASK_SSIZE_T);
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -218,15 +219,15 @@ ssize_t curlx_uztosz(size_t uznum)
 size_t curlx_sotouz(curl_off_t sonum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
   DEBUGASSERT(sonum >= 0);
-  return (size_t)(sonum & (curl_off_t)CURL_MASK_USIZE_T);
+  return (size_t)(sonum & (curl_off_t) CURL_MASK_USIZE_T);
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -237,18 +238,18 @@ size_t curlx_sotouz(curl_off_t sonum)
 int curlx_sztosi(ssize_t sznum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
   DEBUGASSERT(sznum >= 0);
 #if INT_MAX < SSIZE_MAX
-  DEBUGASSERT((size_t)sznum <= (size_t)CURL_MASK_SINT);
+  DEBUGASSERT((size_t) sznum <= (size_t) CURL_MASK_SINT);
 #endif
-  return (int)(sznum & (ssize_t)CURL_MASK_SINT);
+  return (int)(sznum & (ssize_t) CURL_MASK_SINT);
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -259,15 +260,15 @@ int curlx_sztosi(ssize_t sznum)
 unsigned short curlx_uitous(unsigned int uinum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
-  DEBUGASSERT(uinum <= (unsigned int)CURL_MASK_USHORT);
-  return (unsigned short)(uinum & (unsigned int)CURL_MASK_USHORT);
+  DEBUGASSERT(uinum <= (unsigned int) CURL_MASK_USHORT);
+  return (unsigned short) (uinum & (unsigned int) CURL_MASK_USHORT);
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -278,15 +279,15 @@ unsigned short curlx_uitous(unsigned int uinum)
 size_t curlx_sitouz(int sinum)
 {
 #ifdef __INTEL_COMPILER
-#pragma warning(push)
-#pragma warning(disable:810) /* conversion may lose significant bits */
+#  pragma warning(push)
+#  pragma warning(disable:810) /* conversion may lose significant bits */
 #endif
 
   DEBUGASSERT(sinum >= 0);
-  return (size_t)sinum;
+  return (size_t) sinum;
 
 #ifdef __INTEL_COMPILER
-#pragma warning(pop)
+#  pragma warning(pop)
 #endif
 }
 
@@ -316,33 +317,6 @@ bool curlx_sztouz(ssize_t sznum, size_t *puznum)
   return TRUE;
 }
 
-bool curlx_sotouz_fits(curl_off_t sonum, size_t *puznum)
-{
-  if(sonum < 0) {
-    *puznum = 0;
-    return FALSE;
-  }
-#if SIZEOF_CURL_OFF_T > SIZEOF_SIZE_T
-  if(sonum > SIZE_MAX) {
-    *puznum = 0;
-    return FALSE;
-  }
-#endif
-  *puznum = (size_t)sonum;
-  return TRUE;
-}
-
-bool curlx_sltouz(long slnum, size_t *puznum)
-{
-  if(slnum < 0) {
-    *puznum = 0;
-    return FALSE;
-  }
-  /* We error in curl_setup.h if SIZEOF_LONG > SIZEOF_SIZE_T */
-  *puznum = (size_t)slnum;
-  return TRUE;
-}
-
 curl_off_t curlx_uztoso(size_t uznum)
 {
 #if SIZEOF_SIZE_T >= SIZEOF_CURL_OFF_T
