@@ -29,7 +29,6 @@
 #include "multiif.h"
 #include "progress.h"
 #include "curlx/timeval.h"
-#include "curl_printf.h"
 
 /* check rate limits within this many recent milliseconds, at minimum. */
 #define MIN_RATE_LIMIT_PERIOD 3000
@@ -48,7 +47,8 @@ static void time2str(char *r, curl_off_t seconds)
   if(h <= 99) {
     curl_off_t m = (seconds - (h * 3600)) / 60;
     curl_off_t s = (seconds - (h * 3600)) - (m * 60);
-    msnprintf(r, 9, "%2" FMT_OFF_T ":%02" FMT_OFF_T ":%02" FMT_OFF_T, h, m, s);
+    curl_msnprintf(r, 9, "%2" FMT_OFF_T ":%02" FMT_OFF_T ":%02" FMT_OFF_T,
+                   h, m, s);
   }
   else {
     /* this equals to more than 99 hours, switch to a more suitable output
@@ -56,9 +56,9 @@ static void time2str(char *r, curl_off_t seconds)
     curl_off_t d = seconds / 86400;
     h = (seconds - (d * 86400)) / 3600;
     if(d <= 999)
-      msnprintf(r, 9, "%3" FMT_OFF_T "d %02" FMT_OFF_T "h", d, h);
+      curl_msnprintf(r, 9, "%3" FMT_OFF_T "d %02" FMT_OFF_T "h", d, h);
     else
-      msnprintf(r, 9, "%7" FMT_OFF_T "d", d);
+      curl_msnprintf(r, 9, "%7" FMT_OFF_T "d", d);
   }
 }
 
@@ -67,32 +67,27 @@ static void time2str(char *r, curl_off_t seconds)
    Add suffix k, M, G when suitable... */
 static char *max6data(curl_off_t bytes, char *max6)
 {
-  /* a signed 64-bit value is 8192 petabytes maximum */
-  const char unit[] = { 'k', 'M', 'G', 'T', 'P', 0 };
-  int k = 0;
-  if(bytes < 1000000) {
-    msnprintf(max6, 7, "%5" CURL_FORMAT_CURL_OFF_T, bytes);
-    return max6;
+  /* a signed 64-bit value is 8192 petabytes maximum, shown as
+     8.0E (exabytes)*/
+  if(bytes < 100000)
+    curl_msnprintf(max6, 7, "%6" CURL_FORMAT_CURL_OFF_T, bytes);
+  else {
+    const char unit[] = { 'k', 'M', 'G', 'T', 'P', 'E', 0 };
+    int k = 0;
+    curl_off_t nbytes;
+    do {
+      nbytes = bytes / 1024;
+      if(nbytes < 1000)
+        break;
+      bytes = nbytes;
+      k++;
+      DEBUGASSERT(unit[k]);
+    } while(unit[k]);
+    /* xxx.yU */
+    curl_msnprintf(max6, 7, "%3" CURL_FORMAT_CURL_OFF_T
+                   ".%" CURL_FORMAT_CURL_OFF_T "%c", nbytes,
+                   (bytes%1024) / (1024/10), unit[k]);
   }
-
-  do {
-    curl_off_t nbytes = bytes / 1024;
-    if(nbytes < 1000) {
-      /* xxx.yU */
-      msnprintf(max6, 7, "%3" CURL_FORMAT_CURL_OFF_T
-                ".%" CURL_FORMAT_CURL_OFF_T "%c", nbytes,
-                (bytes%1024) / (1024/10), unit[k]);
-      break;
-    }
-    else if(nbytes < 100000) {
-      /* xxxxxU */
-      msnprintf(max6, 7, "%5" CURL_FORMAT_CURL_OFF_T "%c", nbytes, unit[k]);
-      break;
-    }
-    bytes = nbytes;
-    k++;
-    DEBUGASSERT(unit[k]);
-  } while(unit[k]);
   return max6;
 }
 #endif
@@ -122,7 +117,7 @@ int Curl_pgrsDone(struct Curl_easy *data)
   if(!data->progress.hide && !data->progress.callback)
     /* only output if we do not use a progress callback and we are not
      * hidden */
-    fprintf(data->set.err, "\n");
+    curl_mfprintf(data->set.err, "\n");
 
   data->progress.speeder_c = 0; /* reset the progress meter display */
   return 0;
@@ -276,7 +271,7 @@ timediff_t Curl_pgrsLimitWaitTime(struct pgrs_dir *d,
     return 0;
 
   /* The time it took us to have `bytes` */
-  took_ms = curlx_timediff_ceil(now, d->limit.start);
+  took_ms = curlx_timediff_ceil_ms(now, d->limit.start);
 
   /* The time it *should* have taken us to have `bytes`
    * when obeying the bytes_per_sec speed_limit. */
@@ -304,10 +299,9 @@ timediff_t Curl_pgrsLimitWaitTime(struct pgrs_dir *d,
 /*
  * Set the number of downloaded bytes so far.
  */
-CURLcode Curl_pgrsSetDownloadCounter(struct Curl_easy *data, curl_off_t size)
+void Curl_pgrsSetDownloadCounter(struct Curl_easy *data, curl_off_t size)
 {
   data->progress.dl.cur_size = size;
-  return CURLE_OK;
 }
 
 /*
@@ -317,14 +311,14 @@ void Curl_ratelimit(struct Curl_easy *data, struct curltime now)
 {
   /* do not set a new stamp unless the time since last update is long enough */
   if(data->set.max_recv_speed) {
-    if(curlx_timediff(now, data->progress.dl.limit.start) >=
+    if(curlx_timediff_ms(now, data->progress.dl.limit.start) >=
        MIN_RATE_LIMIT_PERIOD) {
       data->progress.dl.limit.start = now;
       data->progress.dl.limit.start_size = data->progress.dl.cur_size;
     }
   }
   if(data->set.max_send_speed) {
-    if(curlx_timediff(now, data->progress.ul.limit.start) >=
+    if(curlx_timediff_ms(now, data->progress.ul.limit.start) >=
        MIN_RATE_LIMIT_PERIOD) {
       data->progress.ul.limit.start = now;
       data->progress.ul.limit.start_size = data->progress.ul.cur_size;
@@ -430,7 +424,7 @@ static bool progress_calc(struct Curl_easy *data, struct curltime now)
       checkindex = (p->speeder_c >= CURR_TIME) ? p->speeder_c%CURR_TIME : 0;
 
       /* Figure out the exact time for the time span */
-      span_ms = curlx_timediff(now, p->speeder_time[checkindex]);
+      span_ms = curlx_timediff_ms(now, p->speeder_time[checkindex]);
       if(span_ms == 0)
         span_ms = 1; /* at least one millisecond MUST have passed */
 
@@ -500,15 +494,15 @@ static void progress_meter(struct Curl_easy *data)
 
   if(!p->headers_out) {
     if(data->state.resume_from) {
-      fprintf(data->set.err,
-              "** Resuming transfer from byte position %" FMT_OFF_T "\n",
-              data->state.resume_from);
+      curl_mfprintf(data->set.err,
+                    "** Resuming transfer from byte position %" FMT_OFF_T "\n",
+                    data->state.resume_from);
     }
-    fprintf(data->set.err,
-            "  %% Total    %% Received %% Xferd  Average Speed   "
-            "Time    Time     Time  Current\n"
-            "                                 Dload  Upload   "
-            "Total   Spent    Left  Speed\n");
+    curl_mfprintf(data->set.err,
+                  "  %% Total    %% Received %% Xferd  Average Speed   "
+                  "Time    Time     Time  Current\n"
+                  "                                 Dload  Upload   "
+                  "Total   Spent    Left  Speed\n");
     p->headers_out = TRUE; /* headers are shown */
   }
 
@@ -542,23 +536,23 @@ static void progress_meter(struct Curl_easy *data)
   /* Get the percentage of data transferred so far */
   total_estm.percent = pgrs_est_percent(total_expected_size, total_cur_size);
 
-  fprintf(data->set.err,
-          "\r"
-          "%3" FMT_OFF_T " %s "
-          "%3" FMT_OFF_T " %s "
-          "%3" FMT_OFF_T " %s %s %s  %s %s %s %s",
-          total_estm.percent, /* 3 letters */           /* total % */
-          max6data(total_expected_size, max6[2]),       /* total size */
-          dl_estm.percent, /* 3 letters */              /* rcvd % */
-          max6data(p->dl.cur_size, max6[0]),            /* rcvd size */
-          ul_estm.percent, /* 3 letters */              /* xfer % */
-          max6data(p->ul.cur_size, max6[1]),            /* xfer size */
-          max6data(p->dl.speed, max6[3]),               /* avrg dl speed */
-          max6data(p->ul.speed, max6[4]),               /* avrg ul speed */
-          time_total,    /* 8 letters */                /* total time */
-          time_spent,    /* 8 letters */                /* time spent */
-          time_left,     /* 8 letters */                /* time left */
-          max6data(p->current_speed, max6[5])
+  curl_mfprintf(data->set.err,
+                "\r"
+                "%3" FMT_OFF_T " %s "
+                "%3" FMT_OFF_T " %s "
+                "%3" FMT_OFF_T " %s %s %s  %s %s %s %s",
+                total_estm.percent, /* 3 letters */         /* total % */
+                max6data(total_expected_size, max6[2]),     /* total size */
+                dl_estm.percent, /* 3 letters */            /* rcvd % */
+                max6data(p->dl.cur_size, max6[0]),          /* rcvd size */
+                ul_estm.percent, /* 3 letters */            /* xfer % */
+                max6data(p->ul.cur_size, max6[1]),          /* xfer size */
+                max6data(p->dl.speed, max6[3]),             /* avrg dl speed */
+                max6data(p->ul.speed, max6[4]),             /* avrg ul speed */
+                time_total,    /* 8 letters */              /* total time */
+                time_spent,    /* 8 letters */              /* time spent */
+                time_left,     /* 8 letters */              /* time left */
+                max6data(p->current_speed, max6[5])
     );
 
   /* we flush the output stream to make it appear as soon as possible */
