@@ -26,7 +26,6 @@
 #include "tool_cfgable.h"
 #include "tool_setopt.h"
 #include "tool_findfile.h"
-#include "tool_formparse.h"
 #include "tool_msgs.h"
 #include "tool_libinfo.h"
 #include "tool_cb_soc.h"
@@ -38,7 +37,6 @@
 #include "tool_cb_see.h"
 #include "tool_cb_dbg.h"
 #include "tool_helpers.h"
-#include "tool_version.h"
 
 #define BUFFER_SIZE 102400L
 
@@ -143,8 +141,8 @@ static CURLcode url_proto_and_rewrite(char **url,
       curl_url_set(uh, CURLUPART_URL, *url,
                    CURLU_GUESS_SCHEME | CURLU_NON_SUPPORT_SCHEME);
     if(!uc) {
-      uc = curl_url_get(uh, CURLUPART_SCHEME, &schemep, CURLU_DEFAULT_SCHEME);
-      if(!uc) {
+      if(!curl_url_get(uh, CURLUPART_SCHEME, &schemep,
+                       CURLU_DEFAULT_SCHEME)) {
 #ifdef CURL_DISABLE_IPFS
         (void)config;
 #else
@@ -164,8 +162,6 @@ static CURLcode url_proto_and_rewrite(char **url,
           proto = proto_token(schemep);
         curl_free(schemep);
       }
-      else if(uc == CURLUE_OUT_OF_MEMORY)
-        result = CURLE_OUT_OF_MEMORY;
     }
     else if(uc == CURLUE_OUT_OF_MEMORY)
       result = CURLE_OUT_OF_MEMORY;
@@ -213,11 +209,11 @@ static CURLcode ssh_setopts(struct OperationConfig *config, CURL *curl)
       config->knownhosts = known;
     }
     else if(!config->hostpubmd5 && !config->hostpubsha256) {
-      errorf("Could not find a known_hosts file");
+      errorf("Couldn't find a known_hosts file");
       return CURLE_FAILED_INIT;
     }
     else
-      warnf("Could not find a known_hosts file");
+      warnf("Couldn't find a known_hosts file");
   }
   return CURLE_OK; /* ignore if SHA256 did not work */
 }
@@ -339,7 +335,8 @@ static CURLcode ssl_setopts(struct OperationConfig *config, CURL *curl)
     MY_SETOPT_STR(curl, CURLOPT_PROXY_CRLFILE, config->crlfile);
 
   if(config->pinnedpubkey) {
-    MY_SETOPT_STR(curl, CURLOPT_PINNEDPUBLICKEY, config->pinnedpubkey);
+    MY_SETOPT_STR(curl, CURLOPT_PINNEDPUBLICKEY,
+                  config->pinnedpubkey);
     if(result)
       warnf("ignoring %s, not supported by libcurl with %s",
             "--pinnedpubkey", ssl_backend());
@@ -416,7 +413,8 @@ static CURLcode ssl_setopts(struct OperationConfig *config, CURL *curl)
   {
     long mask =
       (config->proxy_ssl_allow_beast ? CURLSSLOPT_ALLOW_BEAST : 0) |
-      (config->proxy_ssl_auto_client_cert ? CURLSSLOPT_AUTO_CLIENT_CERT : 0) |
+      (config->proxy_ssl_auto_client_cert ?
+       CURLSSLOPT_AUTO_CLIENT_CERT : 0) |
       (config->proxy_native_ca_store ? CURLSSLOPT_NATIVE_CA : 0);
 
     if(mask)
@@ -484,13 +482,15 @@ static CURLcode ssl_setopts(struct OperationConfig *config, CURL *curl)
 }
 
 /* only called for HTTP transfers */
-static CURLcode http_setopts(struct OperationConfig *config, CURL *curl)
+static CURLcode http_setopts(struct OperationConfig *config,
+                             CURL *curl)
 {
   CURLcode result;
   long postRedir = 0;
 
   my_setopt_long(curl, CURLOPT_FOLLOWLOCATION, config->followlocation);
-  my_setopt_long(curl, CURLOPT_UNRESTRICTED_AUTH, config->unrestricted_auth);
+  my_setopt_long(curl, CURLOPT_UNRESTRICTED_AUTH,
+                 config->unrestricted_auth);
   MY_SETOPT_STR(curl, CURLOPT_AWS_SIGV4, config->aws_sigv4);
   my_setopt_long(curl, CURLOPT_AUTOREFERER, config->autoreferer);
 
@@ -547,8 +547,8 @@ static CURLcode cookie_setopts(struct OperationConfig *config, CURL *curl)
       if(cl == config->cookies)
         result = curlx_dyn_add(&cookies, cl->data);
       else
-        result = curlx_dyn_addf(&cookies, ";%s%s",
-                                ISBLANK(cl->data[0]) ? "" : " ", cl->data);
+        result = curlx_dyn_addf(&cookies, ";%s", cl->data);
+
       if(result) {
         warnf("skipped provided cookie, the cookie header "
               "would go over %u bytes", MAX_COOKIE_LINE);
@@ -586,7 +586,8 @@ static void tcp_setopts(struct OperationConfig *config, CURL *curl)
     my_setopt_long(curl, CURLOPT_TCP_FASTOPEN, 1);
 
   if(config->mptcp)
-    my_setopt_ptr(curl, CURLOPT_OPENSOCKETFUNCTION, tool_socket_open_mptcp_cb);
+    my_setopt_ptr(curl, CURLOPT_OPENSOCKETFUNCTION,
+                  tool_socket_open_mptcp_cb);
 
   /* curl 7.17.1 */
   if(!config->nokeepalive) {
@@ -870,8 +871,7 @@ CURLcode config2setopts(struct OperationConfig *config,
 
   if(proto_http || proto_rtsp) {
     MY_SETOPT_STR(curl, CURLOPT_REFERER, config->referer);
-    MY_SETOPT_STR(curl, CURLOPT_USERAGENT, config->useragent ?
-                  config->useragent : CURL_NAME "/" CURL_VERSION);
+    MY_SETOPT_STR(curl, CURLOPT_USERAGENT, config->useragent);
   }
 
   if(use_proto == proto_http || use_proto == proto_https) {
@@ -941,7 +941,8 @@ CURLcode config2setopts(struct OperationConfig *config,
   my_setopt_long(curl, CURLOPT_FTP_CREATE_MISSING_DIRS,
                  (config->ftp_create_dirs ?
                   CURLFTP_CREATE_DIR_RETRY : CURLFTP_CREATE_DIR_NONE));
-  my_setopt_offt(curl, CURLOPT_MAXFILESIZE_LARGE, config->max_filesize);
+  my_setopt_offt(curl, CURLOPT_MAXFILESIZE_LARGE,
+                 config->max_filesize);
   my_setopt_long(curl, CURLOPT_IPRESOLVE, config->ip_version);
   if(config->socks5_gssapi_nec)
     my_setopt_long(curl, CURLOPT_SOCKS5_GSSAPI_NEC, 1);
